@@ -5,16 +5,57 @@ import json
 import collections
 import traceback
 
+from abc import ABC, abstractmethod
 from pathlib import Path
 from typing_extensions import Literal
-from typing import Dict, Any, Iterable
+from typing import Dict, Any, Iterable, Generic, Sequence, TypeVar, ContextManager, Union
 
 from coba.exceptions import CobaException
 from coba.registry import CobaRegistry
 from coba.utilities import coba_exit
+from coba.pipes import Sink
 
-from coba.config.loggers import Logger
-from coba.config.cachers import Cacher
+_K = TypeVar("_K")
+_V = TypeVar("_V")
+
+class Cacher(Generic[_K, _V], ABC):
+    """The interface for a cacher."""
+    
+    @abstractmethod
+    def __contains__(self, key: _K) -> bool:
+        ...
+
+    @abstractmethod
+    def get(self, key: _K) -> _V:
+        ...
+
+    @abstractmethod
+    def put(self, key: _K, value: _V) -> None:
+        ...
+
+    @abstractmethod
+    def rmv(self, key: _K) -> None:
+        ...
+
+class Logger(ABC):
+    """A more advanced logging interface allowing different types of logs to be written."""
+
+    @property
+    @abstractmethod
+    def sink(self) -> Sink[Iterable[str]]:
+        ...
+
+    @abstractmethod
+    def log(self, message: str) -> 'ContextManager[Logger]':
+        ...
+
+    @abstractmethod
+    def time(self, message: str) -> 'ContextManager[Logger]':
+        ...
+
+    @abstractmethod
+    def log_exception(self, exception:Exception, message: str = "Unexpected exception:") -> None:
+        ...
 
 class ExperimentConfig:
     
@@ -31,18 +72,17 @@ class CobaConfig_meta(type):
     """
 
     def __init__(cls, *args, **kwargs):
-        cls._api_keys   = None
-        cls._cacher     = None
-        cls._logger     = None
-        cls._experiment = None
-
-    @staticmethod
-    def _load_file_configs() -> Dict[str,Any]:
-        search_paths = [Path.home() , Path.cwd(), Path(sys.path[0]) ]
+        cls._api_keys     = None
+        cls._cacher       = None
+        cls._logger       = None
+        cls._experiment   = None
+        cls._global       = {}
+        cls._search_paths = [Path.home() , Path.cwd(), Path(sys.path[0]) ]
 
+    def _load_file_configs(cls) -> Dict[str,Any]:
         config = {}
 
-        for search_path in search_paths:
+        for search_path in cls.search_paths:
 
             potential_coba_config = search_path / ".coba"
 
@@ -86,11 +126,11 @@ class CobaConfig_meta(type):
                     "api_keys"  : collections.defaultdict(lambda:None),
                     "cacher"    : { "DiskCacher": None},
                     "logger"    : { "IndentLogger": "Console" },
-                    "experiment": { "processes": 1, "maxtasksperchild": -1, "chunk_by": "source" }
+                    "experiment": { "processes": 1, "maxtasksperchild": 0, "chunk_by": "source" }
                 }
 
-                for key,value in CobaConfig_meta._load_file_configs().items():
-                    if key in _raw_config and isinstance(_raw_config[key], dict):
+                for key,value in cls._load_file_configs().items():
+                    if key in _raw_config and isinstance(_raw_config[key],dict) and not CobaRegistry.is_known_recipe(value):
                         _raw_config[key].update(value)
                     else:
                         _raw_config[key] = value
@@ -151,6 +191,21 @@ class CobaConfig_meta(type):
     def experiment(cls, value:ExperimentConfig) -> None:
         cls._experiment = value
 
+    @property
+    def store(cls) -> Dict[str,Any]:
+        return cls._global
+
+    @property
+    def search_paths(cls) -> Sequence[str]:
+        return cls._search_paths
+
+    @search_paths.setter
+    def search_paths(cls, value:Union[Sequence[str], Sequence[Path]]) -> None:
+        if len(value) > 0 and isinstance(value[0],str):
+            cls._search_paths = [Path(path) for path in value ]
+        else:
+            cls._search_paths = value
+
 class CobaConfig(metaclass=CobaConfig_meta):
 
     """Create a global configuration context to allow easy mocking and customization.
