@@ -1,15 +1,15 @@
-"""The simulations module contains core classes and types for defining contextual bandit simulations.
+"""The environments module contains core classes and types for defining contextual bandit environments.
 
-This module contains the abstract interface expected for bandit simulations along with the 
-class defining an Interaction within a bandit simulation. Additionally, this module also contains 
+This module contains the abstract interface expected for bandit environments along with the 
+class defining an Interaction within a bandit environment. Additionally, this module also contains 
 the type hints for Context, Action and Reward. These type hints don't contain any functionality. 
-Rather, they simply make it possible to use static type checking for any project that desires 
+They simply make it possible to use static type checking for any project that desires 
 to do so.
-
-TODO Add RegressionSimulation
 """
 
-from coba.environments.core         import Context, Action, SimulatedInteraction, LoggedInteraction, SimulatedEnvironment
+from coba.environments.core         import Context, Action
+from coba.environments.core         import Interaction, SimulatedInteraction, LoggedInteraction
+from coba.environments.core         import Environment, SimulatedEnvironment, LoggedEnvironment, WarmStartEnvironment
 from coba.environments.openml       import OpenmlSource, OpenmlSimulation
 from coba.environments.filters      import SimulationFilter, Sort, Scale, Cycle, Impute
 from coba.environments.pipes        import EnvironmentPipe
@@ -23,9 +23,13 @@ from coba.environments.simulations import (
 __all__ = [
     'Context',
     'Action',
+    'Interaction',
     'SimulatedInteraction',
     'LoggedInteraction',
+    'Environment',
     'SimulatedEnvironment',
+    'LoggedEnvironment',
+    'WarmStartEnvironment',
     'Environments',
     'OpenmlSource',
     'MemorySimulation',
