@@ -1,6 +1,8 @@
+from logging import raiseExceptions
 import re
 import collections
 
+from pathlib import Path
 from copy import copy
 from numbers import Number
 from operator import truediv
@@ -10,8 +12,9 @@ from typing_extensions import Literal
 from typing import Any, Iterable, Dict, List, Tuple, Optional, Sequence, Hashable, Iterator, Union, Type, Callable
 
 from coba.config import CobaConfig
-from coba.utilities import PackageChecker
-from coba.pipes import Filter, Cartesian, JsonEncode, JsonDecode, StopPipe, Pipe, DiskIO
+from coba.exceptions import CobaException
+from coba.utilities import PackageChecker, coba_exit
+from coba.pipes import JsonEncode, JsonDecode, DiskIO, MemoryIO, Sink, Source
 
 class Table:
     """A container class for storing tabular data."""
@@ -92,17 +95,6 @@ class Table:
 
         return [ self._infer_type(column_packed, column_values) for column_packed, column_values in zip(columns_packed,columns_values)]
 
-    def window(self, partition_by:Sequence[str], order_by:Sequence[str], window_lim:Tuple[float,float], functions: Dict[str,Callable]):
-        partitions = collections.defaultdict(list)
-
-        for key in self.keys:
-            flat = self._rows_flat[key]
-            part_key = tuple([flat[part_by] for part_by in partition_by])
-            partitions[part_key].append(flat)
-
-    def aggregate(self, group_by:Sequence[str], functions:Dict[str,Callable]):
-        pass
-
     def filter(self, row_pred:Callable[[Dict[str,Any]],bool] = None, **kwargs) -> 'Table':
 
         def satisifies_filter(col_filter,col_value):
@@ -266,7 +258,7 @@ class InteractionsTable(Table):
                 cumwindow  = list(accumulate(rewards          , lambda a,c: c + (1-alpha)*a))
                 cumdivisor = list(accumulate([1.]*len(rewards), lambda a,c: c + (1-alpha)*a)) #type: ignore
 
-            lrn_sim_rows.append([interactions["learner_id"], interactions["simulation_id"], *list(map(truediv, cumwindow, cumdivisor))])
+            lrn_sim_rows.append([interactions["learner_id"], interactions["environment_id"], *list(map(truediv, cumwindow, cumdivisor))])
 
         if each:
             return lrn_sim_rows
@@ -300,80 +292,249 @@ class InteractionsTable(Table):
         
         if each:
             n_index = len(data[0][2:])
-            return pd.DataFrame(data, columns=["learner_id", "simulation_id", *range(1,n_index+1)])
+            return pd.DataFrame(data, columns=["learner_id", "environment_id", *range(1,n_index+1)])
         
         else:
             n_index = len(data[0][1:])
             return pd.DataFrame(data, columns=["learner_id", *range(1,n_index+1)])
 
+class TransactionIO_V3(Source[Iterable[Any]],Sink[Iterable[Any]]):
+
+    def __init__(self, transaction_log: Optional[str] = None, minify:bool=True) -> None:
+
+        self._io = DiskIO(transaction_log) if transaction_log else MemoryIO()
+        self._minify = minify
+
+    def write(self, items: Iterable[Any]) -> None:
+        if isinstance(self._io, MemoryIO):
+            self._io.write(map(self._encode, items))
+        else:
+            self._io.write(map(JsonEncode(self._minify).filter, map(self._encode,items)))
+
+    def read(self) -> Iterable[Any]:
+        if isinstance(self._io, MemoryIO):
+            return self._io.read()
+        else:
+            return map(JsonDecode().filter,self._io.read())
+
+    @property
+    def result(self) -> 'Result':
+        
+        n_lrns   = None
+        n_sims   = None
+        lrn_rows = {}
+        sim_rows = {}
+        int_rows = {}
+
+        for trx in self.read():
+
+            if trx[0] == "benchmark": 
+                n_lrns = trx[1]["n_learners"]
+                n_sims = trx[1]["n_simulations"]
+            
+            if trx[0] == "S": 
+                sim_rows[trx[1]] = trx[2]
+            
+            if trx[0] == "L": 
+                lrn_rows[trx[1]] = trx[2]
+            
+            if trx[0] == "I": 
+                int_rows[tuple(trx[1])] = trx[2]
+
+        return Result(n_lrns, n_sims, sim_rows, lrn_rows, int_rows)
+
+    def _encode(self,item):
+        if item[0] == "T0":
+            return ['benchmark', {"n_learners":item[1], "n_simulations":item[2]}]
+        
+        if item[0] == "T1":
+            return ["L", item[1], item[2]]
+
+        if item[0] == "T2":
+            return ["S", item[1], item[2]]
+
+        if item[0] == "T3":
+            rows_T = collections.defaultdict(list)
+
+            for row in item[2]:
+                for col,val in row.items():
+                    if col == "rewards" : col="reward"
+                    if col == "reveals" : col="reveal"
+                    rows_T[col].append(val)
+
+            return ["I", item[1], { "_packed": rows_T }]
+
+class TransactionIO_V4(Source[Iterable[Any]],Sink[Iterable[Any]]):
+
+    def __init__(self, transaction_log: Optional[str] = None, minify:bool=True) -> None:
+        self._io = DiskIO(transaction_log) if transaction_log else MemoryIO()
+        self._minify = minify
+
+    def write(self, items: Iterable[Any]) -> None:
+        if isinstance(self._io, MemoryIO):
+            self._io.write(map(self._encode, items))
+        else:
+            self._io.write(map(JsonEncode(self._minify).filter, map(self._encode,items)))
+
+    def read(self) -> Iterable[Any]:
+        if isinstance(self._io, MemoryIO):
+            return self._io.read()
+        else:
+            return map(JsonDecode().filter,self._io.read())
+
+    @property
+    def result(self) -> 'Result':
+        
+        n_lrns   = None
+        n_sims   = None
+        lrn_rows = {}
+        env_rows = {}
+        int_rows = {}
+
+        for trx in self.read():
+
+            if trx[0] == "experiment": 
+                n_lrns = trx[1]["n_learners"]
+                n_sims = trx[1]["n_environments"]
+            
+            if trx[0] == "E": 
+                env_rows[trx[1]] = trx[2]
+            
+            if trx[0] == "L": 
+                lrn_rows[trx[1]] = trx[2]
+            
+            if trx[0] == "I": 
+                int_rows[tuple(trx[1])] = trx[2]
+
+        return Result(n_lrns, n_sims, env_rows, lrn_rows, int_rows)
+
+    def _encode(self,item):
+        if item[0] == "T0":
+            return ['experiment', {"n_learners":item[1], "n_environments":item[2]}]
+        
+        if item[0] == "T1":
+            return ["L", item[1], item[2]]
+
+        if item[0] == "T2":
+            return ["E", item[1], item[2]]
+
+        if item[0] == "T3":
+            rows_T = collections.defaultdict(list)
+
+            for row in item[2]:
+                for col,val in row.items():
+                    if col == "rewards" : col="reward"
+                    if col == "reveals" : col="reveal"
+                    rows_T[col].append(val)
+
+            return ["I", item[1], { "_packed": rows_T }]
+
+        return item
+
+class TransactionIO(Source[Iterable[Any]],Sink[Iterable[Any]]):
+
+    def __init__(self, transaction_log: Optional[str] = None) -> None:
+
+        if not transaction_log or not Path(transaction_log).exists():
+            version = None
+        else:
+            version = JsonDecode().filter(next(DiskIO(transaction_log).read()))[1]
+
+        if version == 3:
+            self._transactionIO = TransactionIO_V3(transaction_log)
+
+        elif version == 4:
+            self._transactionIO = TransactionIO_V4(transaction_log)
+
+        elif version is None:
+            self._transactionIO = TransactionIO_V4(transaction_log)
+            self._transactionIO.write([['version',4]])
+
+        else:
+            raise CobaException("We were unable to determine the appropriate Transaction reader for the file.")
+
+    def write(self, items: Iterable[Any]) -> None:
+        self._transactionIO.write(items)
+
+    def read(self) -> Iterable[Any]:
+        self._transactionIO.read()
+
+    @property
+    def result(self) -> 'Result':
+        return self._transactionIO.result
+
+    def _encode(self,item):
+        if item[0] == "T0":
+            return ['benchmark', {"n_learners":item[1], "n_simulations":item[2]}]
+        
+        if item[0] == "T1":
+            return ["L", item[1], item[2]]
+
+        if item[0] == "T2":
+            return ["S", item[1], item[2]]
+
+        if item[0] == "T3":
+            rows_T = collections.defaultdict(list)
+
+            for row in item[2]:
+                for col,val in row.items():
+                    if col == "rewards" : col="reward"
+                    if col == "reveals" : col="reveal"
+                    rows_T[col].append(val)
+
+            return ["I", item[1], { "_packed": rows_T }]
+
 class Result:
-    """A class representing the result of a Benchmark evaluation on a given collection of Simulations and Learners."""
+    """A class representing the result of an Experiment."""
 
     @staticmethod
     def from_file(filename: str) -> 'Result':
         """Create a Result from a transaction file."""
         
-        #Why is this here??? This is really confusing in practice
-        #if filename is None or not Path(filename).exists(): return Result()
+        if not Path(filename).exists(): 
+            raise CobaException("We were unable to find the given Result file.")
 
-        json_encode = Cartesian(JsonEncode())
-        json_decode = Cartesian(JsonDecode())
-
-        Pipe.join(DiskIO(filename), [json_decode, ResultPromote(), json_encode], DiskIO(filename, append=False)).run()
-        
-        return Result.from_transactions(Pipe.join(DiskIO(filename), [json_decode]).read())
-
-    @staticmethod
-    def from_transactions(transactions: Iterable[Any]) -> 'Result':
-
-        version   = None
-        benchmark = {}
-        lrn_rows  = []
-        sim_rows  = []
-        int_rows  = []
-
-        for trx in transactions:
-            if trx[0] == "version"  : version   = trx[1]
-            if trx[0] == "benchmark": benchmark = trx[1]
-            if trx[0] == "S"        : sim_rows.append({**trx[2], "simulation_id": trx[1]})
-            if trx[0] == "L"        : lrn_rows.append({**trx[2], "learner_id"   : trx[1]})
-            if trx[0] == "I"        : int_rows.append({**trx[2], "simulation_id": trx[1][0], "learner_id": trx[1][1]})
-
-        return Result(version, benchmark, sim_rows, lrn_rows, int_rows)
+        return TransactionIO(filename).result
 
     def __init__(self,
-        version  : Optional[int] = None,
-        benchmark: Dict[str,Any] = {},
-        sim_rows : Sequence[Dict[str,Any]] = [],
-        lrn_rows : Sequence[Dict[str,Any]] = [],
-        int_rows : Sequence[Dict[str,Any]] = []) -> None:
+        n_lrns  : int = None,
+        n_envs  : int = None,
+        env_rows: Dict[int           ,Dict[str,Any]] = {},
+        lrn_rows: Dict[int           ,Dict[str,Any]] = {},
+        int_rows: Dict[Tuple[int,int],Dict[str,Any]] = {}) -> None:
         """Instantiate a Result class."""
 
-        self.version   = version
-        self.benchmark = benchmark
+        self.experiment = {}
 
-        self._simulations  = Table            ("Simulations" , ['simulation_id'              ], sim_rows, ["source"])
-        self._learners     = Table            ("Learners"    , ['learner_id'                 ], lrn_rows, ["family","shuffle","take"])
-        self._interactions = InteractionsTable("Interactions", ['simulation_id', 'learner_id'], int_rows, ["index","reward"])
+        if n_lrns is not None: self.experiment["n_learners"] = n_lrns
+        if n_envs is not None: self.experiment["n_environments"] = n_envs
+
+        env_flat = [ { "environment_id":k,                       **v } for k,v in env_rows.items() ]
+        lrn_flat = [ {                        "learner_id" :k,   **v } for k,v in lrn_rows.items() ]
+        int_flat = [ { "environment_id":k[0], "learner_id":k[1], **v } for k,v in int_rows.items() ]
+
+        self._environments = Table            ("Environments", ['environment_id'              ], env_flat, ["source"])
+        self._learners     = Table            ("Learners"    , ['learner_id'                  ], lrn_flat, ["family","shuffle","take"])
+        self._interactions = InteractionsTable("Interactions", ['environment_id', 'learner_id'], int_flat, ["index","reward"])
 
     @property
     def learners(self) -> Table:
-        """The collection of learners evaluated by Benchmark. The easiest way to work with the 
+        """The collection of learners evaluated by Experiment. The easiest way to work with the 
             learners is to convert them to a pandas data frame via Result.learners.to_pandas()
         """
         return self._learners
 
     @property
-    def simulations(self) -> Table:
-        """The collection of simulations used to evaluate each learner in the Benchmark. The easiest
-            way to work with simulations is to convert to a dataframe via Result.simulations.to_pandas()
+    def environments(self) -> Table:
+        """The collection of environments used to evaluate each learner in the Experiment. The easiest
+            way to work with environments is to convert to a dataframe via Result.environments.to_pandas()
         """
-        return self._simulations
+        return self._environments
 
     @property
     def interactions(self) -> InteractionsTable:
-        """The collection of interactions that learners chose actions for in the Benchmark. Each interaction
-            has a simulation_id and learner_id column to link them to the learners and simulations tables. The 
+        """The collection of interactions that learners chose actions for in the Experiment. Each interaction
+            has a environment_id and learner_id column to link them to the learners and environments tables. The 
             easiest way to work with interactions is to convert to a dataframe via Result.interactions.to_pandas()
         """
         return self._interactions
@@ -381,7 +542,7 @@ class Result:
     def _copy(self) -> 'Result':
         result = Result()
 
-        result.simulations  = copy(self._simulations)
+        result.environments = copy(self._environments)
         result.learners     = copy(self._learners)
         result.interactions = copy(self._interactions)
 
@@ -393,22 +554,22 @@ class Result:
             return all((sim_id, lrn_id) in self.interactions for lrn_id in self.learners.keys)
 
         new_result               = copy(self)
-        new_result._simulations  = self.simulations.filter(simulation_id=is_complete_sim)
-        new_result._interactions = self.interactions.filter(simulation_id=is_complete_sim)
+        new_result._environments = self.environments.filter(environment_id=is_complete_sim)
+        new_result._interactions = self.interactions.filter(environment_id=is_complete_sim)
 
-        if len(new_result.simulations) == 0:
+        if len(new_result.environments) == 0:
             CobaConfig.logger.log(f"No simulation was found with interaction data for every learner.")
 
         return new_result
 
-    def filter_sim(self, pred:Callable[[Dict[str,Any]],bool] = None, **kwargs) -> 'Result':
+    def filter_env(self, pred:Callable[[Dict[str,Any]],bool] = None, **kwargs) -> 'Result':
 
         new_result = copy(self)
-        new_result._simulations  = new_result.simulations.filter(pred, **kwargs)
-        new_result._interactions = new_result.interactions.filter(simulation_id=new_result.simulations)
+        new_result._environments = new_result.environments.filter(pred, **kwargs)
+        new_result._interactions = new_result.interactions.filter(environment_id=new_result.environments)
 
-        if len(new_result.simulations) == 0:
-            CobaConfig.logger.log(f"No simulations matched the given filter: {kwargs}.")
+        if len(new_result.environments) == 0:
+            CobaConfig.logger.log(f"No environments matched the given filter: {kwargs}.")
 
         return new_result
 
@@ -430,8 +591,8 @@ class Result:
         each : bool = False,
         filename: str = None,
         ax = None) -> None:
-        """This plots the performance of multiple Learners on multiple simulations. It gives a sense of the expected 
-            performance for different learners across independent simulations. This plot is valuable in gaining insight 
+        """This plots the performance of multiple learners on multiple environments. It gives a sense of the expected 
+            performance for different learners across independent environments. This plot is valuable in gaining insight 
             into how various learners perform in comparison to one another. 
 
         Args:
@@ -538,152 +699,7 @@ class Result:
             plt.savefig(filename, dpi=300)
 
     def __str__(self) -> str:
-        return str({ "Learners": len(self._learners), "Simulations": len(self._simulations), "Interactions": len(self._interactions) })
+        return str({ "Learners": len(self._learners), "Environments": len(self._environments), "Interactions": len(self._interactions) })
 
     def __repr__(self) -> str:
         return str(self)
-
-class ResultPromote(Filter):
-
-    CurrentVersion = 3
-
-    def filter(self, items: Iterable[Any]) -> Iterable[Any]:
-        items_iter = iter(items)
-        items_peek = next(items_iter)
-        items_iter = chain([items_peek], items_iter)
-
-        version = 0 if items_peek[0] != 'version' else items_peek[1]
-
-        if version == ResultPromote.CurrentVersion:
-            raise StopPipe()
-
-        while version != ResultPromote.CurrentVersion:
-            if version == 0:
-                promoted_items = [["version",1]]
-
-                for transaction in items:
-
-                    if transaction[0] == "L":
-
-                        index  = transaction[1][1]['learner_id']
-                        values = transaction[1][1]
-
-                        del values['learner_id']
-
-                        promoted_items.append([transaction[0], index, values])
-
-                    if transaction[0] == "S":
-
-                        index  = transaction[1][1]['simulation_id']
-                        values = transaction[1][1]
-
-                        del values['simulation_id']
-
-                        promoted_items.append([transaction[0], index, values])
-
-                    if transaction[0] == "B":
-                        key_columns = ['learner_id', 'simulation_id', 'seed', 'batch_index']
-                        
-                        index  = [ transaction[1][1][k] for k in key_columns ]
-                        values = transaction[1][1]
-                        
-                        for key_column in key_columns: del values[key_column]
-                        
-                        if 'reward' in values:
-                            values['reward'] = values['reward'].estimate
-                        
-                        if 'mean_reward' in values:
-                            values['reward'] = values['mean_reward'].estimate
-                            del values['mean_reward']
-
-                        values['reward'] = round(values['reward', 5])
-
-                        promoted_items.append([transaction[0], index, values])
-
-                items   = promoted_items
-                version = 1
-
-            if version == 1:
-
-                n_seeds       : Optional[int]                  = None
-                S_transactions: Dict[int, Any]                 = {}
-                S_seeds       : Dict[int, List[Optional[int]]] = collections.defaultdict(list)
-
-                B_rows: Dict[Tuple[int,int], Dict[str, List[float]] ] = {}
-                B_cnts: Dict[int, int                               ] = {}
-
-                promoted_items = [["version",2]]
-
-                for transaction in items:
-
-                    if transaction[0] == "benchmark":
-                        n_seeds = transaction[1].get('n_seeds', None)
-
-                        del transaction[1]['n_seeds']
-                        del transaction[1]['batcher']
-                        del transaction[1]['ignore_first']
-
-                        promoted_items.append(transaction)
-
-                    if transaction[0] == "L":
-                        promoted_items.append(transaction)
-
-                    if transaction[0] == "S":
-                        S_transactions[transaction[1]] == transaction
-
-                    if transaction[0] == "B":
-                        S_id = transaction[1][1]
-                        seed = transaction[1][2]
-                        L_id = transaction[1][0]
-                        B_id = transaction[1][3]
-                        
-                        if n_seeds is None:
-                            raise StopPipe("We are unable to promote logs from version 1 to version 2")
-
-                        if seed not in S_seeds[S_id]:
-                            S_seeds[S_id].append(seed)
-                            
-                            new_S_id = n_seeds * S_id + S_seeds[S_id].index(seed)
-                            new_dict = S_transactions[S_id][2].clone()
-                            
-                            new_dict["source"]  = str(S_id)
-                            new_dict["filters"] = f'[{{"Shuffle":{seed}}}]'
-
-                            B_cnts[S_id] = new_dict['batch_count']
-
-                            promoted_items.append(["S", new_S_id, new_dict])
-
-                        if B_id == 0: B_rows[(S_id, L_id)] = {"N":[], "reward":[]}
-
-                        B_rows[(S_id, L_id)]["N"     ].append(transaction[2]["N"])
-                        B_rows[(S_id, L_id)]["reward"].append(transaction[2]["reward"])
-
-                        if len(B_rows[(S_id, L_id)]["N"]) == B_cnts[S_id]:
-                            promoted_items.append(["B", [S_id, L_id], B_rows[(S_id, L_id)]])
-                            del B_rows[(S_id, L_id)]
-
-                items   = promoted_items
-                version = 2
-
-            if version == 2:
-
-                promoted_items = [["version",3]]
-
-                for transaction in items:
-                    
-                    #upgrade all reward entries to the packed format which will now allow array types and dict types.
-                    if transaction[0] == "B":
-                        rewards = transaction[2]["reward"]
-                        del transaction[2]["reward"]
-                        transaction[2]["_packed"] = {"reward": rewards}
-                    
-                    #Change from B to I to be consistent with result property name: `interactions`
-                    if transaction[0] == "B": 
-                        transaction[0] = "I"
-                    
-                    promoted_items.append(transaction)
-
-                items   = promoted_items
-                version = 3
-
-        return items
