@@ -1,4 +1,5 @@
 from copy import deepcopy
+from multiprocessing.synchronize import Lock, Semaphore
 from multiprocessing import Manager
 from threading import Thread
 from typing import Sequence, Iterable, Any
@@ -11,10 +12,12 @@ class CobaMultiprocessFilter(Filter[Iterable[Any], Iterable[Any]]):
 
     class ConfiguredFilter:
 
-        def __init__(self, filters: Sequence[Filter], logger_sink: Sink, with_name:bool) -> None:
+        def __init__(self, filters: Sequence[Filter], logger_sink: Sink, with_name:bool, manager) -> None:
 
-            self._logger = deepcopy(CobaConfig.logger)
-            self._cacher = deepcopy(CobaConfig.cacher)
+            self._logger    = deepcopy(CobaConfig.logger)
+            self._cacher    = deepcopy(CobaConfig.cacher)
+            self._srcsema   = manager.Semaphore(2)
+            self._cachelock = manager.Lock()
 
             if isinstance(self._logger, IndentLogger):
                 self._logger._with_name = with_name
@@ -29,15 +32,17 @@ class CobaMultiprocessFilter(Filter[Iterable[Any], Iterable[Any]]):
         def filter(self, item: Iterable[Any]) -> Iterable[Any]:
 
             #placing this here means this is only set inside the process 
-            CobaConfig.logger = self._logger
-            CobaConfig.cacher = self._cacher
+            CobaConfig.logger            = self._logger
+            CobaConfig.cacher            = self._cacher
+            CobaConfig.store["srcsema"]  = self._srcsema
+            CobaConfig.store["cachelck"] = self._cachelock
 
             return Pipe.join(self._filters).filter(item)
 
-    def __init__(self, filters: Sequence[Filter], processes=1, maxtasksperchild=None) -> None:
+    def __init__(self, filters: Sequence[Filter], processes=1, maxtasksperchild=0) -> None:
         self._filters          = filters
         self._processes        = processes
-        self._maxtasksperchild = None if maxtasksperchild == -1 else maxtasksperchild
+        self._maxtasksperchild = maxtasksperchild
 
     def filter(self, items: Iterable[Any]) -> Iterable[Any]:
 
@@ -55,13 +60,17 @@ class CobaMultiprocessFilter(Filter[Iterable[Any], Iterable[Any]]):
                             CobaConfig.logger.log_exception(err)
                         else:
                             CobaConfig.logger.log_exception(err[2])
-                            print("".join(err[3]))
+                            #err[3] contains the stack trace... 
+                            #I'm not sure what to do with it at this point, but here it is if any wants it in the future
+                            #When we pass our exception back to the original thread it loses its stacktrace so if we want
+                            #to report stack trace we'll need to turn it into a string and pass it with the exception???
+                            #print("".join(err[3]))
 
                 log_thread = Thread(target=log_stderr)
                 log_thread.daemon = True
                 log_thread.start()
 
-                filter = CobaMultiprocessFilter.ConfiguredFilter(self._filters, stderr, self._processes>1)
+                filter = CobaMultiprocessFilter.ConfiguredFilter(self._filters, stderr, self._processes>1, manager)
 
                 for item in MultiprocessFilter([filter], self._processes, self._maxtasksperchild, stderr).filter(items):
                     yield item
