@@ -1,7 +1,5 @@
-"""The data.filters module contains core classes for filters used in data pipelines.
+"""The coba.pipes.filters module contains core classes for filters used in data pipelines."""
 
-TODO add docstrings for all filters
-"""
 import re
 import csv
 import collections
@@ -31,13 +29,13 @@ _T_Data       = Union[_T_DenseData, _T_SparseData]
 class Cartesian(Filter[Union[Any,Iterable[Any]], Iterable[Any]]):
 
     def __init__(self, filter: Union[Filter,Sequence[Filter]]):
-        
+
         self._filters = filter if isinstance(filter, collections.Sequence) else [filter]
 
     def filter(self, item: Union[Any,Iterable[Any]]) -> Iterable[Any]:
 
         items = item if isinstance(item, collections.Iterable) else [item]
-        
+
         for item in items:
             for filter in self._filters:
                 yield filter.filter(item)
@@ -45,6 +43,9 @@ class Cartesian(Filter[Union[Any,Iterable[Any]], Iterable[Any]]):
 class Identity(Filter[Any, Any]):
     def filter(self, item:Any) -> Any:
         return item
+    
+    def __repr__(self) -> str:
+        return "{ Identity }"
 
 class Shuffle(Filter[Iterable[Any], Iterable[Any]]):
 
@@ -109,7 +110,7 @@ class Take(Filter[Iterable[Any], Iterable[Any]]):
             first    = [next(items)] if self._keep_first else []
             resevoir = list(islice(items,self._count))
 
-            if self._seed is not None:            
+            if self._seed is not None:
                 rng = CobaRandom(self._seed)
                 W = 1
 
@@ -123,7 +124,7 @@ class Take(Filter[Iterable[Any], Iterable[Any]]):
                     pass
 
             return itertools.chain( first, resevoir if len(resevoir) == self._count else [])
-        
+
     def __repr__(self) -> str:
         return str(self.params)
 
@@ -149,22 +150,37 @@ class ResponseToLines(Filter[Response, Iterable[str]]):
         return item.content.decode('utf-8').split('\n')
 
 class JsonEncode(Filter[Any, str]):
+ 
+    def _min(self,obj):
+        #WARNING: This method doesn't handle primitive types such int, float, or str. We handle this shortcoming
+        #WARNING: by making sure no primitive type is passed to this method in filter. Accepting the shortcoming
+        #WARNING: improves the performance of this method by a few percentage points. 
 
-    def _intify(self,obj):
-
-        if isinstance(obj,float) and obj.is_integer():
-            return int(obj)
+        #JsonEncoder writes floats with .0 regardless of if they are integers so we convert them to int to save space
+        #JsonEncoder also writes floats out 16 digits so we round down to 5 here to reduce file size
 
         if isinstance(obj,tuple):
             obj = list(obj)
+            kv  = enumerate(obj) 
+        elif isinstance(obj,list):
+            kv = enumerate(obj)
+        elif isinstance(obj,dict):
+            kv = obj.items()
+        else:
+            return obj
 
-        if isinstance(obj,list):
-            for i in range(len(obj)):
-                obj[i] = self._intify(obj[i])
-
-        if isinstance(obj,dict):
-            for key in obj:
-                obj[key] = self._intify(obj[key])
+        for k,v in kv:
+            if isinstance(v, (int,str)):
+                obj[k] = v
+            elif isinstance(v, float):
+                if v.is_integer():
+                    obj[k] = int(v) 
+                elif math.isnan(v) or math.isinf(v):
+                    obj[k] = v                    
+                else: 
+                    obj[k] = f"|{v:0.5g}|" #rounding by any means is considerably slower than this crazy method
+            else:
+                obj[k] = self._min(v)
 
         return obj
 
@@ -176,13 +192,8 @@ class JsonEncode(Filter[Any, str]):
         else:
             self._encoder = CobaJsonEncoder()
 
-    def filter(self, item: Any) -> str:
-        if self._minify:
-            #JsonEncoder writes floats with .0 regardless of if they are integers
-            #Therefore we preprocess and turn all float whole numbers into integers
-            return self._encoder.encode(self._intify(item))
-        else:
-            return self._encoder.encode(item)
+    def filter(self, item: Any) -> str:        
+        return self._encoder.encode(self._min([item])[0] if self._minify else item).replace('"|',"").replace('|"',"")
 
 class JsonDecode(Filter[str, Any]):
     def __init__(self, decoder: json.decoder.JSONDecoder = CobaJsonDecoder()) -> None:
@@ -196,7 +207,7 @@ class ArffReader(Filter[Iterable[str], _T_Data]):
         https://waikato.github.io/weka-wiki/formats_and_processing/arff_stable/
     """
 
-    def __init__(self, skip_encoding: Union[bool,Sequence[Union[str,int]]] = False):
+    def __init__(self, skip_encoding: Union[bool,Sequence[Union[str,int]]] = False, **dialect):
 
         self._skip_encoding = skip_encoding
 
@@ -218,6 +229,8 @@ class ArffReader(Filter[Iterable[str], _T_Data]):
         #The @data line indicates when the data begins. After @data there should be no more @ lines.
         self._r_data = re.compile(r'^@[Dd][Aa][Tt][Aa]')
 
+        self._dialect = dialect
+
     def _determine_encoder(self, index:int, name: str, tipe: str) -> Encoder:
 
         is_numeric = tipe in ['numeric', 'integer', 'real']
@@ -251,7 +264,7 @@ class ArffReader(Filter[Iterable[str], _T_Data]):
                 if attribute_match:
                     attribute_text  = attribute_match.group(1).strip()
                     attribute_type  = re.split('[ ]', attribute_text, 1)[1]
-                    attribute_name  = re.split('[ ]', attribute_text)[0].lower()
+                    attribute_name  = re.split('[ ]', attribute_text)[0]
                     attribute_index = len(headers)
 
                     headers.append(attribute_name)
@@ -265,7 +278,7 @@ class ArffReader(Filter[Iterable[str], _T_Data]):
             if in_data_section and line != '':
                 data_lines.append(line)
 
-        parsed_data = CsvReader(True).filter(itertools.chain([",".join(headers)], data_lines))
+        parsed_data = CsvReader(True, **self._dialect).filter(itertools.chain([",".join(headers)], data_lines))
 
         return parsed_data, encoders
 
@@ -277,15 +290,16 @@ class ArffReader(Filter[Iterable[str], _T_Data]):
 
 class CsvReader(Filter[Iterable[str], _T_Data]):
 
-    def __init__(self, has_header: bool):
+    def __init__(self, has_header: bool, **dialect):
         self._has_header = has_header
+        self._dialect = dialect
 
     def filter(self, items: Iterable[str]) -> _T_Data:
 
-        lines = iter(filter(None, csv.reader(i.strip() for i in items)))
+        lines = iter(filter(None, csv.reader( (i.strip() for i in items), **self._dialect)))
 
         try:
-            header     = [ h.lower() for h in next(lines)] if self._has_header else None
+            header     = [ h.strip().strip('\'"') for h in next(lines)] if self._has_header else None
             first_data = next(lines)
         except StopIteration:
             return [header] #[None] because every other filter method assumes there is some kind of a header row.
@@ -302,7 +316,8 @@ class CsvReader(Filter[Iterable[str], _T_Data]):
         yield header if header is None else OrderedDict(zip(header, itertools.count()))
 
         for line in lines:
-            yield OrderedDict((int(k),v) for l in line for k,v in [l.strip("}{").strip().split(' ', 1)])
+            if len(line) > 1 or line[0] != '{}':
+                yield OrderedDict((int(k),v) for l in line for k,v in [l.strip("}{").strip().split(' ', 1)])
 
 class LibSvmReader(Filter[Iterable[str], _T_SparseData]):
 
@@ -327,7 +342,6 @@ class ManikReader(Filter[Iterable[str], _T_SparseData]):
     """http://manikvarma.org/downloads/XC/XMLRepository.html"""
     """https://drive.google.com/file/d/1u7YibXAC_Wz1RDehN1KjB5vu21zUnapV/view"""
 
-
     def filter(self, lines: Iterable[str]) -> _T_SparseData:
 
         # we skip first line because it just has metadata
@@ -343,7 +357,7 @@ class Encode(Filter[_T_Data, _T_Data]):
     def filter(self, items: _T_Data) -> _T_Data:
 
         items  = iter(items) # this makes sure items are pulled out for fitting
-        
+
         if not self._has_header:
             header_synced_encoders = self._encoders
 
@@ -399,28 +413,24 @@ class Drop(Filter[_T_Data, _T_Data]):
             header = next(data)
 
             if header is None:
-                drop_keys = self._drop_cols
-                yield header
-            
+                drop_keys = sorted(self._drop_cols,reverse=True)
             elif isinstance(header,dict):
-                drop_keys = [ header.pop(k) for k in self._drop_cols ]
-                yield header
-            
+                drop_keys = sorted([ header.pop(k) for k in self._drop_cols ],reverse=True)
             elif isinstance(header,list):
-                drop_keys = [ header.index(k) for k in self._drop_cols ]
-                for i in sorted(drop_keys,reverse=True): header.pop(i)
-                yield header 
-            
+                drop_keys = sorted([ header.index(k) for k in self._drop_cols ],reverse=True)
+                for i in drop_keys: header.pop(i)
             else:
                 raise CobaException(f"Unrecognized type ({type(header).__name__}) passed to Drops.")
 
+            yield header
+
             for row in data:
 
                 if self._drop_row and self._drop_row(row):
                     continue
 
-                for k in sorted(drop_keys,reverse=True):
-                    row.pop(k)                
+                for k in drop_keys:
+                    row.pop(k)
 
                 yield row
 
@@ -430,7 +440,7 @@ class Structure(Filter[_T_Data, Iterable[Any]]):
         self._col_structure = split_cols
 
     def filter(self, data: _T_Data) -> _T_Data:
-        
+
         data   = iter(data)
         header = next(data)
 
